import pandas as pd
pd.options.display.max_colwidth = 1000
import textwrap
class Reviews:
    def __init__(self, df, review_titles, prop_colname="Proposal Name", reviewer_colname="Reviewer Name", str_wrap_len=35) -> None:
        self.df = df
        self.prop_colname = prop_colname
        self.reviewer_colname = reviewer_colname
        self.review_titles = review_titles
        self.str_wrap_len = str_wrap_len

    def get_all_review_sub(self, reviewer, proposal):
        ret = []
        for title in self.review_titles:
            sent = self.df[(self.df[self.prop_colname] == proposal) & (self.df[self.reviewer_colname] == reviewer)][title]
            if len(sent) != 0:
                ret.append(sent.to_string(index=False))
            else:
                ret.append("NaN")
        return ret

    def wrap(self, string):
        return '\n'.join(textwrap.wrap(string, self.str_wrap_len))
    
    def get_reviews_in_order(self, reviewer, proposal, order):
        ret = []
        for i in range(len(order)):
            if order[i] in self.df.columns:
                sent = self.df[(self.df[self.prop_colname] == proposal) & (self.df[self.reviewer_colname] == reviewer)][order[i]]
                if len(sent) != 0:
                    ret.append(self.wrap(sent.to_string(index=False)))
                else:
                    ret.append("NaN")
        return ret


from tkinter import *

class legend_window:
    def __init__(self, dict, gui, all_ratings) -> None:
        self.gui = gui
        self.root2 = Tk()
        self.root2.title('Legend Window for Rankings UI')
        self.root2.geometry('700x500+100+150')
        
        self.dict = dict
        self.graph_attr = list(dict.keys())
        self.ratings = all_ratings + ["None"]
        self.var_list = []
        
        for i in range(len(self.graph_attr)):
            tkvarq = StringVar(self.root2)
            tkvarq.set(self.dict[self.graph_attr[i]])
            self.var_list.append(tkvarq)
            self.create_wigets(self.graph_attr[i], i)

        submit_button = Button(self.root2, text="Submit and Reopen the Main Window", command=self.return_pairs)
        paddings = {'padx': 5, 'pady': 5}
        submit_button.grid(column=1, row=len(self.graph_attr)+1)
        
    def create_wigets(self, ga, order):
        # padding for widgets using the grid layout
        paddings = {'padx': 5, 'pady': 5}

        # label
        label = Label(self.root2, text=f'Please Select the Rating Displayed by the Graphical Attribute of {ga}:')
        label.grid(column=0, row=order, sticky=W, **paddings)

        # option menu
        option_menu = OptionMenu(
            self.root2,
            self.var_list[order],
            *self.ratings)
        option_menu.grid(column=1, row=order, sticky=W, **paddings)

    def return_pairs(self):
        self.result = {}
        pairs = list(i.get() for i in self.var_list if i.get() != "None")
        #if len(set(pairs)) == len(pairs):
        for i in range(len(self.var_list)):
            self.result[self.graph_attr[i]] = self.var_list[i].get()
        self.gui.update_all_rects(self.result)
        self.root2.destroy()

    def show(self):
        self.root2.mainloop()

    def get_pair(self):
        return self.result



import pandas as pd

class Reviewers:
    def __init__(self, df) -> None:
        self.df = df

    def get_name(self, email):
        return self.df.loc[self.df["Reviewer Email"] == email, "Reviewer Name"].iloc[0]
    

    def get_number(self, email):
        return self.df.loc[self.df["Reviewer Email"] == email, "Reviewer Number"].iloc[0]


import pandas as pd
import numpy as np

class Rankings:
    def __init__(self, rating_df, ranking, ties, rating_names, reviewer_col_name="Reviewer Name", prop_col_name="Proposal Name", overall_col_name="Overall Score"):
        self.scores = rating_df
        self.ranking = ranking
        self.ties = ties
        self.rating_names = rating_names
        self.reviewer_col_name = reviewer_col_name
        self.overall_col_name = overall_col_name
        self.proposal_col_name = prop_col_name
        self.index = list(self.scores[self.reviewer_col_name].unique())
        self.columns = list(self.scores[self.proposal_col_name].unique())
        self.num_papers = len(self.columns)

    def get_rating_df(self, rat_name):
        df = pd.DataFrame(index=self.scores[self.reviewer_col_name].unique(), columns=self.scores[self.proposal_col_name].unique())
        for index in self.index:
            for column in self.columns:
                df.loc[index][column] = self.get_sub_rating(rat_name, index, column)
        return df

    def updated_pairs(self, dict, yesno, topk):
        df = self.scores.copy()
        if dict is not None:
            for key in dict.keys():
                df = df[(dict[key][0] <= df[key]) & (df[key] <= dict[key][1])]
        if yesno is not None:
            for key in yesno.keys():
                df = df[(df[key] == yesno[key])]
        df = df[[self.reviewer_col_name, self.proposal_col_name]]
        dict_r = self.get_all_rankings(topk=topk)
        rank_list = [(i,x) for i in dict_r for x in dict_r[i]]
        filter_list = list(df.itertuples(index=False, name=None))
        return list(set(rank_list).intersection(set(filter_list)))

    def get_name(self, email):
        return self.scores.loc[self.scores["Reviewer Email"] == email, self.reviewer_col_name].iloc[0]

    def get_sub_rating(self, rat_name, reviewer, prop):
        l = self.scores.loc[(self.scores[self.reviewer_col_name] == reviewer) & (self.scores[self.proposal_col_name] == prop), rat_name].tolist()
        if l:
            return str(l[0])
        else:
            return np.nan

    def get_all_sub_ratings(self):
        return list(self.rating_names)

    def get_columns(self):
        return list(self.index)

    def get_all_rankings(self, topk=None):
        ret = {}
        for key in self.index:
            papers = self.ranking[key]
            selected = papers
            if topk is not None:
                selected = papers[:topk]
                if self.ties is not None:
                    for tie in self.ties[key]:
                        for i in range(len(tie)):
                            if selected[-1] == tie[i]:
                                for other in tie:
                                    if other not in selected:
                                        selected.append(other)
                                break
            ret[key] = selected
        return ret

    def get_op_rankings(self):
        df_op = self.get_rating_df(self.overall_col_name)
        ret = {}
        rows = list(self.index)
        props = list(self.columns)
        maxi = 0
        for reviewer in rows:
            ret[reviewer] = {}
            for prop in props:
                rate = df_op.loc[reviewer, prop]
                if not pd.isna(rate):
                    if rate not in ret[reviewer]:
                        ret[reviewer][rate] = [prop]
                    else:
                        ret[reviewer][rate].append(prop)
                        if len(ret[reviewer][rate]) > maxi:
                            maxi = len(ret[reviewer][rate])
        return ret, maxi


from tkinter import *
from tkinter import ttk
import tkinter as tk
from tkinter import messagebox

class filter_window:
    def __init__(self, gui, numerical_attr, yes_no_attr, score_range, num_papers, defalut_dict=None) -> None:
        self.gui = gui
        self.root2 = Tk()
        self.root2.title('Filter Window for Rankings UI')
        self.root2.geometry('500x400+100+150')
        self.score_range = score_range
        self.num_papers = num_papers
        self.dict = numerical_attr
        self.yesno_list = yes_no_attr
        self.ratings = list(numerical_attr.keys())
        self.var_min = []
        self.var_max = []

        self.yesno_var = []
        self.default_dict = defalut_dict
        self.tkvarq_topk = StringVar(self.root2)
        self.tkvarq_topk.set("Please Select a Topk")
        for i in range(len(self.ratings)):
            tkvarq_max = StringVar(self.root2)
            tkvarq_min = StringVar(self.root2)
            tkvarq_max.set("Please Select a Maximum")
            tkvarq_min.set("Please Select a Minimum")
            self.var_max.append(tkvarq_max)
            self.var_min.append(tkvarq_min)
            self.create_wigets(self.ratings[i], i)

        for i in range(len(yes_no_attr)):
            tkvarq_yesno = StringVar(self.root2)
            tkvarq_yesno.set("Please Select Yes/No")
            self.yesno_var.append(tkvarq_yesno)
            self.create_wigets_yesno(i)

        submit_button = Button(self.root2, text="Submit and Reopen the Main Window", command=self.return_pairs)
        paddings = {'padx': 2, 'pady': 5}
        submit_button.grid(column=1, row=len(self.ratings)+3)
        label_min = ttk.Label(self.root2, text=f'Minimum Score')
        label_min.grid(column=1, row=0, sticky=tk.W, **paddings)
        label_max = ttk.Label(self.root2, text=f'Maximum Score')
        label_max.grid(column=2, row=0, sticky=tk.W, **paddings)

        label_topk = ttk.Label(self.root2, text=f'Top-K Rankings:')
        label_topk.grid(column=0, row=len(self.ratings)+len(self.yesno_list)+1, sticky=tk.W, **paddings)
        option_menu_topk = ttk.OptionMenu(
            self.root2,
            self.tkvarq_topk,
            str(self.default_dict["topk"]),
            *range(1, self.num_papers+1))
        option_menu_topk.grid(column=1, row=len(self.ratings)+len(self.yesno_list)+1, sticky=tk.W, **paddings)

    def create_wigets_yesno(self, order):
        # padding for widgets using the grid layout
        paddings = {'padx': 2, 'pady': 5}

        # label
        label_yesno = ttk.Label(self.root2, text=self.yesno_list[order])
        label_yesno.grid(column=0, row=len(self.ratings)+order+1, sticky=tk.W, **paddings)
        option_menu_req = ttk.OptionMenu(
            self.root2,
            self.yesno_var[order],
            self.default_dict[self.yesno_list[order]],
            *["Yes", "No"])
        option_menu_req.grid(column=1, row=len(self.ratings)+order+1, sticky=tk.W, **paddings)

    def create_wigets(self, label, order):
        # padding for widgets using the grid layout
        paddings = {'padx': 2, 'pady': 5}
        # label
        label1 = ttk.Label(self.root2, text=f'{label}:')
        label1.grid(column=0, row=order+1, sticky=tk.W, **paddings)
        option_menu_max = ttk.OptionMenu(
            self.root2,
            self.var_max[order],
            str(max(self.default_dict[label])),
            *self.score_range)
        option_menu_min = ttk.OptionMenu(
            self.root2,
            self.var_min[order],
            str(min(self.default_dict[label])),
            *self.score_range)

        option_menu_max.grid(column=2, row=order+1, sticky=tk.W, **paddings)
        option_menu_min.grid(column=1, row=order+1, sticky=tk.W, **paddings)


    def return_pairs(self):
        self.result = {}
        self.yesno_ret = {}
        valid = True
        for i in range(len(self.ratings)):
            min = self.var_min[i].get()
            max = self.var_max[i].get()
            if int(min) > int(max):
                messagebox.showerror(title="Illegal Values", message="Minimun cannot be larger than the maximum.")     
                valid = False
                break 
            self.result[self.ratings[i]] = [int(self.var_min[i].get()), int(self.var_max[i].get())]
        for j in range(len(self.yesno_list)):
            self.yesno_ret[self.yesno_list[j]] = str(self.yesno_var[j].get())
        if valid:
            self.gui.filter_ratings(self.result, self.yesno_ret, int(self.tkvarq_topk.get()))
            self.root2.destroy()
    
    def show(self):
        self.root2.mainloop()


